@@ -231,10 +231,7 @@ ProxyAccessible::Bounds()
   if (FAILED(hr)) {
     return rect;
   }
-  rect.x = left;
-  rect.y = top;
-  rect.width = width;
-  rect.height = height;
+  rect.SetRect(left, top, width, height);
   return rect;
 }
 
@@ -487,7 +484,7 @@ GetIA2TextBoundary(AccessibleTextBoundary aGeckoBoundaryType)
     case nsIAccessibleText::BOUNDARY_LINE_START:
       return IA2_TEXT_BOUNDARY_LINE;
     default:
-      MOZ_RELEASE_ASSERT(false);
+      MOZ_CRASH();
   }
 }
 
@@ -677,7 +674,7 @@ ProxyAccessible::ScrollSubstringToPoint(int32_t aStartOffset, int32_t aEndOffset
   } else if (aCoordinateType == nsIAccessibleCoordinateType::COORDTYPE_PARENT_RELATIVE) {
     coordType = IA2_COORDTYPE_PARENT_RELATIVE;
   } else {
-    MOZ_RELEASE_ASSERT(false, "unsupported coord type");
+    MOZ_CRASH("unsupported coord type");
   }
 
   acc->scrollSubstringToPoint(static_cast<long>(aStartOffset),
