@@ -7,8 +7,6 @@
 ChromeUtils.import("resource:///modules/SitePermissions.jsm");
 ChromeUtils.import("resource://gre/modules/BrowserUtils.jsm");
 
-const nsIQuotaManagerService = Ci.nsIQuotaManagerService;
-
 var gPermURI;
 var gPermPrincipal;
 var gUsageRequest;
@@ -73,10 +71,6 @@ function initRow(aPartId) {
     state = SitePermissions.getDefault(aPartId);
   }
   setRadioState(aPartId, state);
-
-  if (aPartId == "indexedDB") {
-    initIndexedDBRow();
-  }
 }
 
 function createRow(aPartId) {
@@ -162,65 +156,3 @@ function setRadioState(aPartId, aValue) {
     radio.radioGroup.selectedItem = radio;
   }
 }
-
-function initIndexedDBRow() {
-  // IndexedDB information is not shown for pages with a null principal
-  // such as sandboxed pages because these pages do not have access to indexedDB.
-  if (gPermPrincipal.isNullPrincipal)
-    return;
-
-  let row = document.getElementById("perm-indexedDB-row");
-  let extras = document.getElementById("perm-indexedDB-extras");
-
-  row.appendChild(extras);
-
-  var quotaManagerService =
-    Cc["@mozilla.org/dom/quota-manager-service;1"]
-      .getService(nsIQuotaManagerService);
-  gUsageRequest =
-    quotaManagerService.getUsageForPrincipal(gPermPrincipal,
-                                             onIndexedDBUsageCallback);
-
-  var status = document.getElementById("indexedDBStatus");
-  var button = document.getElementById("indexedDBClear");
-
-  status.value = "";
-  status.setAttribute("hidden", "true");
-  button.setAttribute("hidden", "true");
-}
-
-function onIndexedDBClear() {
-  Cc["@mozilla.org/dom/quota-manager-service;1"]
-    .getService(nsIQuotaManagerService)
-    .clearStoragesForPrincipal(gPermPrincipal);
-
-  Cc["@mozilla.org/serviceworkers/manager;1"]
-    .getService(Ci.nsIServiceWorkerManager)
-    .removeAndPropagate(gPermURI.host);
-
-  SitePermissions.remove(gPermURI, "indexedDB");
-  initIndexedDBRow();
-}
-
-function onIndexedDBUsageCallback(request) {
-  let uri = request.principal.URI;
-  if (!uri.equals(gPermURI)) {
-    throw new Error("Callback received for bad URI: " + uri);
-  }
-
-  let usage = request.result.usage;
-  if (usage) {
-    if (!("DownloadUtils" in window)) {
-      ChromeUtils.import("resource://gre/modules/DownloadUtils.jsm");
-    }
-
-    var status = document.getElementById("indexedDBStatus");
-    var button = document.getElementById("indexedDBClear");
-
-    status.value =
-      gBundle.getFormattedString("indexedDBUsage",
-                                 DownloadUtils.convertByteUnits(usage));
-    status.removeAttribute("hidden");
-    button.removeAttribute("hidden");
-  }
-}
