@@ -23,13 +23,14 @@
 #include "nsContentUtils.h"
 #include "nsError.h"
 #include "nsHostObjectURI.h"
+#include "nsIAsyncShutdown.h"
 #include "nsIException.h" // for nsIStackFrame
 #include "nsIMemoryReporter.h"
 #include "nsIPrincipal.h"
 #include "nsIUUIDGenerator.h"
 #include "nsNetUtil.h"
 
-#define RELEASING_TIMER 1000
+#define RELEASING_TIMER 5000
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -49,18 +50,21 @@ struct DataInfo
     : mObjectType(eBlobImpl)
     , mBlobImpl(aBlobImpl)
     , mPrincipal(aPrincipal)
+    , mRevoked(false)
   {}
 
   DataInfo(DOMMediaStream* aMediaStream, nsIPrincipal* aPrincipal)
     : mObjectType(eMediaStream)
     , mMediaStream(aMediaStream)
     , mPrincipal(aPrincipal)
+    , mRevoked(false)
   {}
 
   DataInfo(MediaSource* aMediaSource, nsIPrincipal* aPrincipal)
     : mObjectType(eMediaSource)
     , mMediaSource(aMediaSource)
     , mPrincipal(aPrincipal)
+    , mRevoked(false)
   {}
 
   ObjectType mObjectType;
@@ -74,12 +78,17 @@ struct DataInfo
 
   // WeakReferences of nsHostObjectURI objects.
   nsTArray<nsWeakPtr> mURIs;
+
+  // When a blobURL is revoked, we keep it alive for RELEASING_TIMER
+  // milliseconds in order to support pending operations such as navigation,
+  // download and so on.
+  bool mRevoked;
 };
 
 static nsClassHashtable<nsCStringHashKey, DataInfo>* gDataTable;
 
 static DataInfo*
-GetDataInfo(const nsACString& aUri)
+GetDataInfo(const nsACString& aUri, bool aAlsoIfRevoked = false)
 {
   if (!gDataTable) {
     return nullptr;
@@ -106,6 +115,10 @@ GetDataInfo(const nsACString& aUri)
     gDataTable->Get(StringHead(aUri, pos), &res);
   }
 
+  if (!aAlsoIfRevoked && res && res->mRevoked) {
+    return nullptr;
+  }
+
   return res;
 }
 
@@ -155,9 +168,8 @@ BroadcastBlobURLRegistration(const nsACString& aURI,
 }
 
 void
-BroadcastBlobURLUnregistration(const nsACString& aURI, DataInfo* aInfo)
+BroadcastBlobURLUnregistration(const nsCString& aURI)
 {
-  MOZ_ASSERT(aInfo);
   MOZ_ASSERT(NS_IsMainThread());
 
   if (XRE_IsParentProcess()) {
@@ -166,8 +178,7 @@ BroadcastBlobURLUnregistration(const nsACString& aURI, DataInfo* aInfo)
   }
 
   dom::ContentChild* cc = dom::ContentChild::GetSingleton();
-  Unused << NS_WARN_IF(!cc->SendUnstoreAndBroadcastBlobURLUnregistration(
-    nsCString(aURI)));
+  Unused << NS_WARN_IF(!cc->SendUnstoreAndBroadcastBlobURLUnregistration(aURI));
 }
 
 class HostObjectURLsReporter final : public nsIMemoryReporter
@@ -424,34 +435,50 @@ NS_IMPL_ISUPPORTS(BlobURLsReporter, nsIMemoryReporter)
 
 class ReleasingTimerHolder final : public nsITimerCallback
                                  , public nsINamed
+                                 , public nsIAsyncShutdownBlocker
 {
 public:
   NS_DECL_ISUPPORTS
 
   static void
-  Create(nsTArray<nsWeakPtr>&& aArray)
+  Create(const nsACString& aURI, bool aBroadcastToOtherProcesses)
   {
-    RefPtr<ReleasingTimerHolder> holder = new ReleasingTimerHolder(Move(aArray));
+    MOZ_ASSERT(NS_IsMainThread());
+
+    RefPtr<ReleasingTimerHolder> holder =
+      new ReleasingTimerHolder(aURI, aBroadcastToOtherProcesses);
+
+    auto raii = mozilla::MakeScopeExit([&] {
+      holder->CancelTimerAndRevokeURI();
+    });
+
     nsresult rv = NS_NewTimerWithCallback(getter_AddRefs(holder->mTimer),
                                           holder, RELEASING_TIMER,
                                           nsITimer::TYPE_ONE_SHOT,
                                           SystemGroup::EventTargetFor(TaskCategory::Other));
     NS_ENSURE_SUCCESS_VOID(rv);
+
+    nsCOMPtr<nsIAsyncShutdownClient> phase = GetShutdownPhase();
+    NS_ENSURE_TRUE_VOID(!!phase);
+
+    rv = phase->AddBlocker(holder, NS_LITERAL_STRING(__FILE__), __LINE__,
+                           NS_LITERAL_STRING("ReleasingTimerHolder shutdown"));
+    NS_ENSURE_SUCCESS_VOID(rv);
+
+    raii.release();
   }
 
+  // nsITimerCallback interface
+
   NS_IMETHOD
   Notify(nsITimer* aTimer) override
   {
-    for (uint32_t i = 0; i < mURIs.Length(); ++i) {
-      nsCOMPtr<nsIURI> uri = do_QueryReferent(mURIs[i]);
-      if (uri) {
-        static_cast<nsHostObjectURI*>(uri.get())->ForgetBlobImpl();
-      }
-    }
-
+    RevokeURI(mBroadcastToOtherProcesses);
     return NS_OK;
   }
 
+  // nsINamed interface
+
   NS_IMETHOD
   GetName(nsACString& aName) override
   {
@@ -459,19 +486,105 @@ public:
     return NS_OK;
   }
 
+  // nsIAsyncShutdownBlocker interface
+
+  NS_IMETHOD
+  GetName(nsAString& aName) override
+  {
+    aName.AssignLiteral("ReleasingTimerHolder");
+    return NS_OK;
+  }
+
+  NS_IMETHOD
+  BlockShutdown(nsIAsyncShutdownClient* aClient) override
+  {
+    CancelTimerAndRevokeURI();
+    return NS_OK;
+  }
+
+  NS_IMETHOD
+  GetState(nsIPropertyBag**) override
+  {
+    return NS_OK;
+  }
+
 private:
-  explicit ReleasingTimerHolder(nsTArray<nsWeakPtr>&& aArray)
-    : mURIs(aArray)
+  ReleasingTimerHolder(const nsACString& aURI, bool aBroadcastToOtherProcesses)
+    : mURI(aURI)
+    , mBroadcastToOtherProcesses(aBroadcastToOtherProcesses)
   {}
 
   ~ReleasingTimerHolder()
   {}
 
-  nsTArray<nsWeakPtr> mURIs;
+  void
+  RevokeURI(bool aBroadcastToOtherProcesses)
+  {
+    // Remove the shutting down blocker
+    nsCOMPtr<nsIAsyncShutdownClient> phase = GetShutdownPhase();
+    if (phase) {
+      phase->RemoveBlocker(this);
+    }
+
+    // If we have to broadcast the unregistration, let's do it now.
+    if (aBroadcastToOtherProcesses) {
+      BroadcastBlobURLUnregistration(mURI);
+    }
+
+    DataInfo* info = GetDataInfo(mURI, true /* We care about revoked dataInfo */);
+    if (!info) {
+      // Already gone!
+      return;
+    }
+
+    MOZ_ASSERT(info->mRevoked);
+
+    for (uint32_t i = 0; i < info->mURIs.Length(); ++i) {
+      nsCOMPtr<nsIURI> uri = do_QueryReferent(info->mURIs[i]);
+      if (uri) {
+        static_cast<nsHostObjectURI*>(uri.get())->ForgetBlobImpl();
+      }
+    }
+
+    gDataTable->Remove(mURI);
+    if (gDataTable->Count() == 0) {
+      delete gDataTable;
+      gDataTable = nullptr;
+    }
+  }
+
+  void
+  CancelTimerAndRevokeURI()
+  {
+    if (mTimer) {
+      mTimer->Cancel();
+      mTimer = nullptr;
+    }
+
+    RevokeURI(false /* aBroadcastToOtherProcesses */);
+  }
+
+  static nsCOMPtr<nsIAsyncShutdownClient>
+  GetShutdownPhase()
+  {
+    nsCOMPtr<nsIAsyncShutdownService> svc = services::GetAsyncShutdown();
+    NS_ENSURE_TRUE(!!svc, nullptr);
+
+    nsCOMPtr<nsIAsyncShutdownClient> phase;
+    nsresult rv = svc->GetXpcomWillShutdown(getter_AddRefs(phase));
+    NS_ENSURE_SUCCESS(rv, nullptr);
+
+    return Move(phase);
+  }
+
+  nsCString mURI;
+  bool mBroadcastToOtherProcesses;
+
   nsCOMPtr<nsITimer> mTimer;
 };
 
-NS_IMPL_ISUPPORTS(ReleasingTimerHolder, nsITimerCallback, nsINamed)
+NS_IMPL_ISUPPORTS(ReleasingTimerHolder, nsITimerCallback, nsINamed,
+                  nsIAsyncShutdownBlocker)
 
 } // namespace mozilla
 
@@ -593,7 +706,8 @@ nsHostObjectProtocolHandler::GetAllBlobURLEntries(
     }
 
     aRegistrations.AppendElement(BlobURLRegistrationData(
-      nsCString(iter.Key()), ipcBlob, IPC::Principal(info->mPrincipal)));
+      nsCString(iter.Key()), ipcBlob, IPC::Principal(info->mPrincipal),
+                info->mRevoked));
   }
 
   return true;
@@ -612,26 +726,19 @@ nsHostObjectProtocolHandler::RemoveDataEntry(const nsACString& aUri,
     return;
   }
 
-  if (aBroadcastToOtherProcesses && info->mObjectType == DataInfo::eBlobImpl) {
-    BroadcastBlobURLUnregistration(aUri, info);
-  }
+  info->mRevoked = true;
 
-  if (!info->mURIs.IsEmpty()) {
-    ReleasingTimerHolder::Create(Move(info->mURIs));
-  }
-
-  gDataTable->Remove(aUri);
-  if (gDataTable->Count() == 0) {
-    delete gDataTable;
-    gDataTable = nullptr;
-  }
+  // The timer will take care of removing the entry for real after
+  // RELEASING_TIMER milliseconds. In the meantime, the DataInfo, marked as
+  // revoked, will not be exposed.
+  ReleasingTimerHolder::Create(aUri,
+                               aBroadcastToOtherProcesses &&
+                                 info->mObjectType == DataInfo::eBlobImpl);
 }
 
 /* static */ void
 nsHostObjectProtocolHandler::RemoveDataEntries()
 {
-  MOZ_ASSERT(XRE_IsContentProcess());
-
   if (!gDataTable) {
     return;
   }
@@ -792,7 +899,6 @@ nsHostObjectProtocolHandler::NewURI(const nsACString& aSpec,
   rv = uri->SetSpec(aSpec);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  NS_TryToSetImmutable(uri);
   uri.forget(aResult);
 
   if (info && info->mObjectType == DataInfo::eBlobImpl) {
@@ -865,7 +971,7 @@ nsHostObjectProtocolHandler::NewChannel2(nsIURI* uri,
   nsCOMPtr<nsIChannel> channel;
   rv = NS_NewInputStreamChannelInternal(getter_AddRefs(channel),
                                         uri,
-                                        stream,
+                                        stream.forget(),
                                         NS_ConvertUTF16toUTF8(contentType),
                                         EmptyCString(), // aContentCharset
                                         aLoadInfo);
@@ -930,6 +1036,18 @@ nsFontTableProtocolHandler::GetScheme(nsACString &result)
   return NS_OK;
 }
 
+/* static */ void
+nsHostObjectProtocolHandler::StoreClonedURI(const nsACString& aSpec,
+                                            nsIURI* aURI)
+{
+  MOZ_ASSERT(aURI);
+
+  DataInfo* info = GetDataInfo(aSpec);
+  if (info) {
+    info->mURIs.AppendElement(do_GetWeakReference(aURI));
+  }
+}
+
 nsresult
 NS_GetBlobForBlobURI(nsIURI* aURI, BlobImpl** aBlob)
 {
