@@ -187,6 +187,8 @@ nsHostObjectURI::CloneInternal(mozilla::net::nsSimpleURI::RefHandlingEnum aRefHa
   u->mPrincipal = mPrincipal;
   u->mBlobImpl = mBlobImpl;
 
+  nsHostObjectProtocolHandler::StoreClonedURI(newRef, simpleClone);
+
   simpleClone.forget(aClone);
   return NS_OK;
 }
@@ -244,18 +246,18 @@ nsHostObjectURI::GetScriptableHelper(nsIXPCScriptable **_retval)
 }
 
 NS_IMETHODIMP
-nsHostObjectURI::GetContractID(char * *aContractID)
+nsHostObjectURI::GetContractID(nsACString& aContractID)
 {
   // Make sure to modify any subclasses as needed if this ever
   // changes.
-  *aContractID = nullptr;
+  aContractID.SetIsVoid(true);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHostObjectURI::GetClassDescription(char * *aClassDescription)
+nsHostObjectURI::GetClassDescription(nsACString& aClassDescription)
 {
-  *aClassDescription = nullptr;
+  aClassDescription.SetIsVoid(true);
   return NS_OK;
 }
 
