@@ -237,9 +237,6 @@ private:
   void SendHangNotification(const HangData& aHangData,
                             const nsString& aBrowserDumpId);
 
-  void OnTakeFullMinidumpComplete(const HangData& aHangData,
-                                  const nsString& aDumpId);
-
   void ClearHangNotification();
 
   void ForcePaintOnThread(TabId aTabId, uint64_t aLayerObserverEpoch);
@@ -647,20 +644,11 @@ HangMonitorParent::SendHangNotification(const HangData& aHangData,
   MOZ_RELEASE_ASSERT(NS_IsMainThread());
 
   // We already have a full minidump; go ahead and use it.
-  OnTakeFullMinidumpComplete(aHangData, aBrowserDumpId);
-}
-
-void
-HangMonitorParent::OnTakeFullMinidumpComplete(const HangData& aHangData,
-                                              const nsString& aDumpId)
-{
-  mProcess->SetHangData(aHangData, aDumpId);
+  mProcess->SetHangData(aHangData, aBrowserDumpId);
 
   nsCOMPtr<nsIObserverService> observerService =
     mozilla::services::GetObserverService();
-  observerService->NotifyObservers(mProcess,
-                                   "process-hang-report",
-                                   nullptr);
+  observerService->NotifyObservers(mProcess, "process-hang-report", nullptr);
 }
 
 void
