@@ -169,14 +169,15 @@ public:
   }
 };
 
-ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(nsIPrincipal* aPrincipal,
-                                               const nsACString& aScope,
-                                               const nsACString& aScriptSpec,
-                                               nsILoadGroup* aLoadGroup,
-                                               nsLoadFlags aLoadFlags)
+ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(
+    nsIPrincipal* aPrincipal,
+    const nsACString& aScope,
+    const nsACString& aScriptSpec,
+    nsILoadGroup* aLoadGroup,
+    ServiceWorkerUpdateViaCache aUpdateViaCache)
   : ServiceWorkerJob(Type::Update, aPrincipal, aScope, aScriptSpec)
   , mLoadGroup(aLoadGroup)
-  , mLoadFlags(aLoadFlags)
+  , mUpdateViaCache(aUpdateViaCache)
 {
 }
 
@@ -188,15 +189,16 @@ ServiceWorkerUpdateJob::GetRegistration() const
   return ref.forget();
 }
 
-ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(Type aType,
-                                               nsIPrincipal* aPrincipal,
-                                               const nsACString& aScope,
-                                               const nsACString& aScriptSpec,
-                                               nsILoadGroup* aLoadGroup,
-                                               nsLoadFlags aLoadFlags)
+ServiceWorkerUpdateJob::ServiceWorkerUpdateJob(
+    Type aType,
+    nsIPrincipal* aPrincipal,
+    const nsACString& aScope,
+    const nsACString& aScriptSpec,
+    nsILoadGroup* aLoadGroup,
+    ServiceWorkerUpdateViaCache aUpdateViaCache)
   : ServiceWorkerJob(aType, aPrincipal, aScope, aScriptSpec)
   , mLoadGroup(aLoadGroup)
-  , mLoadFlags(aLoadFlags)
+  , mUpdateViaCache(aUpdateViaCache)
 {
 }
 
@@ -327,10 +329,10 @@ ServiceWorkerUpdateJob::Update()
   }
 }
 
-nsLoadFlags
-ServiceWorkerUpdateJob::GetLoadFlags() const
+ServiceWorkerUpdateViaCache
+ServiceWorkerUpdateJob::GetUpdateViaCache() const
 {
-  return mLoadFlags;
+  return mUpdateViaCache;
 }
 
 void
@@ -381,8 +383,6 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
     }
   }
 
-  mLoadFlags = aLoadFlags;
-
   nsAutoCString defaultAllowedPrefix;
   rv = GetRequiredScopeStringPrefix(scriptURI, defaultAllowedPrefix,
                                     eUseDirectory);
@@ -429,13 +429,17 @@ ServiceWorkerUpdateJob::ComparisonResult(nsresult aStatus,
   Telemetry::Accumulate(Telemetry::SERVICE_WORKER_UPDATED, 1);
 
   // Begin step 7 of the Update algorithm to evaluate the new script.
+  nsLoadFlags flags = aLoadFlags;
+  if (GetUpdateViaCache() == ServiceWorkerUpdateViaCache::None) {
+    flags |= nsIRequest::VALIDATE_ALWAYS;
+  }
 
   RefPtr<ServiceWorkerInfo> sw =
     new ServiceWorkerInfo(mRegistration->mPrincipal,
                           mRegistration->mScope,
                           mScriptSpec,
                           aNewCacheName,
-                          mLoadFlags);
+                          flags);
 
   mRegistration->SetEvaluating(sw);
 
