@@ -163,9 +163,29 @@ function* run_test_1(generator)
   // Check that the cookie service accepted the new cookie.
   Assert.equal(Services.cookiemgr.countCookiesFromHost(cookie.host), 1);
 
-  // Wait for the cookie service to rename the old database and rebuild.
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
+  let isRebuildingDone = false;
+  let rebuildingObserve = function (subject, topic, data) {
+    isRebuildingDone = true;
+    Services.obs.removeObserver(rebuildingObserve, "cookie-db-rebuilding");
+  };
+  Services.obs.addObserver(rebuildingObserve, "cookie-db-rebuilding");
+
+  // Crash test: we're going to rebuild the cookie database. Close all the db
+  // connections in the main thread and initialize a new database file in the
+  // cookie thread. Trigger some access of cookies to ensure we won't crash in
+  // the chaos status.
+  for (let i = 0; i < 10; ++i) {
+    Assert.equal(Services.cookiemgr.countCookiesFromHost(cookie.host), 1)
+    executeSoon(function() { do_run_generator(sub_generator); });
+    yield;
+  }
+
+  // Wait for the cookie service to rename the old database and rebuild if not yet.
+  if (!isRebuildingDone) {
+    Services.obs.removeObserver(rebuildingObserve, "cookie-db-rebuilding");
+    new _observer(sub_generator, "cookie-db-rebuilding");
+    yield;
+  }
   executeSoon(function() { do_run_generator(sub_generator); });
   yield;
 
@@ -232,20 +252,9 @@ function* run_test_2(generator)
   // succeeded.
   Assert.ok(!do_get_backup_file(profile).exists());
 
-  // Synchronously read in the first cookie. This will cause it to go into the
-  // cookie table, whereupon it will be written out during database rebuild.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-
-  // Wait for the asynchronous read to choke, at which point the backup file
-  // will be created and the database rebuilt.
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
-  executeSoon(function() { do_run_generator(sub_generator); });
-  yield;
-
-  // At this point, the cookies should still be in memory.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-  Assert.equal(do_count_cookies(), 1);
+  // Recreate a new database since it was corrupted
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 0);
+  Assert.equal(do_count_cookies(), 0);
 
   // Close the profile.
   do_close_profile(sub_generator);
@@ -255,13 +264,11 @@ function* run_test_2(generator)
   Assert.ok(do_get_backup_file(profile).exists());
   Assert.equal(do_get_backup_file(profile).fileSize, size);
   let db = Services.storage.openDatabase(do_get_cookie_file(profile));
-  Assert.equal(do_count_cookies_in_db(db, "0.com"), 1);
   db.close();
 
-  // Load the profile, and check that it contains the new cookie.
   do_load_profile();
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-  Assert.equal(do_count_cookies(), 1);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 0);
+  Assert.equal(do_count_cookies(), 0);
 
   // Close the profile.
   do_close_profile(sub_generator);
@@ -310,25 +317,16 @@ function* run_test_3(generator)
   // succeeded.
   Assert.ok(!do_get_backup_file(profile).exists());
 
-  // Synchronously read in the cookies for our two domains. The first should
-  // succeed, but the second should fail midway through, resulting in none of
-  // those cookies being present.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("hither.com"), 10);
+  // Recreate a new database since it was corrupted
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("hither.com"), 0);
   Assert.equal(Services.cookiemgr.countCookiesFromHost("haithur.com"), 0);
 
-  // Wait for the backup file to be created and the database rebuilt.
-  Assert.ok(!do_get_backup_file(profile).exists());
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
-  executeSoon(function() { do_run_generator(sub_generator); });
-  yield;
-
   // Close the profile.
   do_close_profile(sub_generator);
   yield;
   let db = Services.storage.openDatabase(do_get_cookie_file(profile));
-  Assert.equal(do_count_cookies_in_db(db, "hither.com"), 10);
-  Assert.equal(do_count_cookies_in_db(db), 10);
+  Assert.equal(do_count_cookies_in_db(db, "hither.com"), 0);
+  Assert.equal(do_count_cookies_in_db(db), 0);
   db.close();
 
   // Check that the original database was renamed.
@@ -346,13 +344,6 @@ function* run_test_3(generator)
   // Synchronously read in everything.
   Assert.equal(do_count_cookies(), 0);
 
-  // Wait for the backup file to be created and the database rebuilt.
-  Assert.ok(!do_get_backup_file(profile).exists());
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
-  executeSoon(function() { do_run_generator(sub_generator); });
-  yield;
-
   // Close the profile.
   do_close_profile(sub_generator);
   yield;
@@ -397,26 +388,17 @@ function* run_test_4(generator)
   // succeeded.
   Assert.ok(!do_get_backup_file(profile).exists());
 
-  // Synchronously read in the first cookie. This will cause it to go into the
-  // cookie table, whereupon it will be written out during database rebuild.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
+  // Recreate a new database since it was corrupted
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 0);
 
   // Queue up an INSERT for the same base domain. This should also go into
   // memory and be written out during database rebuild.
   let uri = NetUtil.newURI("http://0.com/");
   Services.cookies.setCookieString(uri, null, "oh2=hai; max-age=1000", null);
 
-  // Wait for the asynchronous read to choke and the insert to fail shortly
-  // thereafter, at which point the backup file will be created and the database
-  // rebuilt.
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
-  executeSoon(function() { do_run_generator(sub_generator); });
-  yield;
-
   // At this point, the cookies should still be in memory.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 2);
-  Assert.equal(do_count_cookies(), 2);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
+  Assert.equal(do_count_cookies(), 1);
 
   // Close the profile.
   do_close_profile(sub_generator);
@@ -425,14 +407,11 @@ function* run_test_4(generator)
   // Check that the original database was renamed.
   Assert.ok(do_get_backup_file(profile).exists());
   Assert.equal(do_get_backup_file(profile).fileSize, size);
-  let db = Services.storage.openDatabase(do_get_cookie_file(profile));
-  Assert.equal(do_count_cookies_in_db(db, "0.com"), 2);
-  db.close();
 
   // Load the profile, and check that it contains the new cookie.
   do_load_profile();
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 2);
-  Assert.equal(do_count_cookies(), 2);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
+  Assert.equal(do_count_cookies(), 1);
 
   // Close the profile.
   do_close_profile(sub_generator);
@@ -474,22 +453,10 @@ function* run_test_5(generator)
   // succeeded.
   Assert.ok(!do_get_backup_file(profile).exists());
 
-  // Synchronously read in the first two cookies. This will cause them to go
-  // into the cookie table, whereupon it will be written out during database
-  // rebuild.
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("bar.com"), 1);
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-
-  // Wait for the asynchronous read to choke, at which point the backup file
-  // will be created and a new connection opened.
-  new _observer(sub_generator, "cookie-db-rebuilding");
-  yield;
-
-  // At this point, the cookies should still be in memory. (Note that these
-  // calls are re-entrant into the cookie service, but it's OK!)
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("bar.com"), 1);
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-  Assert.equal(do_count_cookies(), 2);
+  // Recreate a new database since it was corrupted
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("bar.com"), 0);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 0);
+  Assert.equal(do_count_cookies(), 0);
   Assert.ok(do_get_backup_file(profile).exists());
   Assert.equal(do_get_backup_file(profile).fileSize, size);
   Assert.ok(!do_get_rebuild_backup_file(profile).exists());
@@ -502,39 +469,24 @@ function* run_test_5(generator)
   Assert.equal(do_count_cookies_in_db(db.db), 1);
   db.close();
 
-  // Wait for the rebuild to bail and the database to be closed.
-  new _observer(sub_generator, "cookie-db-closed");
-  yield;
-
   // Check that the original backup and the database itself are gone.
-  Assert.ok(do_get_rebuild_backup_file(profile).exists());
   Assert.ok(do_get_backup_file(profile).exists());
   Assert.equal(do_get_backup_file(profile).fileSize, size);
-  Assert.ok(!do_get_cookie_file(profile).exists());
 
-  // Check that the rebuild backup has the original bar.com cookie, and possibly
-  // a 0.com cookie depending on whether it got written out first or second.
-  db = new CookieDatabaseConnection(do_get_rebuild_backup_file(profile), 4);
-  Assert.equal(do_count_cookies_in_db(db.db, "bar.com"), 1);
-  let count = do_count_cookies_in_db(db.db);
-  Assert.ok(count == 1 ||
-    count == 2 && do_count_cookies_in_db(db.db, "0.com") == 1);
-  db.close();
-
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("bar.com"), 1);
-  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 1);
-  Assert.equal(do_count_cookies(), 2);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("bar.com"), 0);
+  Assert.equal(Services.cookiemgr.countCookiesFromHost("0.com"), 0);
+  Assert.equal(do_count_cookies(), 0);
 
   // Close the profile. We do not need to wait for completion, because the
-  // database has already been closed.
-  do_close_profile();
+  // database has already been closed. Ensure the cookie file is unlocked.
+  do_close_profile(sub_generator);
+  yield;
 
   // Clean up.
+  do_get_cookie_file(profile).remove(false);
   do_get_backup_file(profile).remove(false);
-  do_get_rebuild_backup_file(profile).remove(false);
   Assert.ok(!do_get_cookie_file(profile).exists());
   Assert.ok(!do_get_backup_file(profile).exists());
-  Assert.ok(!do_get_rebuild_backup_file(profile).exists());
   do_run_generator(generator);
 }
 
