@@ -202,7 +202,7 @@ VerifyRGBXCorners(uint8_t* aData, const IntSize &aSize, const int32_t aStride, S
   const int pixelSize = 4;
   MOZ_ASSERT(aSize.width * pixelSize <= aStride);
 
-  const int translation = bounds.y * aStride + bounds.x * pixelSize;
+  const int translation = bounds.Y() * aStride + bounds.X() * pixelSize;
   const int topLeft = translation;
   const int topRight = topLeft + (width - 1) * pixelSize;
   const int bottomLeft = translation + (height - 1) * aStride;
@@ -220,7 +220,7 @@ VerifyRGBXCorners(uint8_t* aData, const IntSize &aSize, const int32_t aStride, S
         int column = (offset % aStride) / pixelSize;
         gfxCriticalError() << "RGBX corner pixel at (" << column << "," << row << ") in "
                            << aSize.width << "x" << aSize.height << " surface, bounded by "
-                           << "(" << bounds.x << "," << bounds.y << "," << width << ","
+                           << "(" << bounds.X() << "," << bounds.Y() << "," << width << ","
                            << height << ") is not opaque: "
                            << int(aData[offset]) << ","
                            << int(aData[offset+1]) << ","
@@ -541,7 +541,7 @@ static void SetPaintPattern(SkPaint& aPaint, const Pattern& aPattern,
           aPaint.setColor(SK_ColorTRANSPARENT);
           break;
         }
-        mat.preTranslate(pat.mSamplingRect.x, pat.mSamplingRect.y);
+        mat.preTranslate(pat.mSamplingRect.X(), pat.mSamplingRect.Y());
       }
 
       SkShader::TileMode xTileMode = ExtendModeToTileMode(pat.mExtendMode, Axis::X_AXIS);
@@ -896,16 +896,16 @@ ShrinkClippedStrokedRect(const Rect &aStrokedRect, const IntRect &aDeviceClip,
   Rect userSpaceStrokeClip =
     UserSpaceStrokeClip(aDeviceClip, aTransform, aStrokeOptions);
   RectDouble strokedRectDouble(
-    aStrokedRect.x, aStrokedRect.y, aStrokedRect.Width(), aStrokedRect.Height());
+    aStrokedRect.X(), aStrokedRect.Y(), aStrokedRect.Width(), aStrokedRect.Height());
   RectDouble intersection =
-    strokedRectDouble.Intersect(RectDouble(userSpaceStrokeClip.x,
-                                           userSpaceStrokeClip.y,
+    strokedRectDouble.Intersect(RectDouble(userSpaceStrokeClip.X(),
+                                           userSpaceStrokeClip.Y(),
                                            userSpaceStrokeClip.Width(),
                                            userSpaceStrokeClip.Height()));
   Double dashPeriodLength = DashPeriodLength(aStrokeOptions);
   if (intersection.IsEmpty() || dashPeriodLength == 0.0f) {
-    return Rect(
-      intersection.x, intersection.y, intersection.Width(), intersection.Height());
+    return Rect(intersection.X(), intersection.Y(),
+                intersection.Width(), intersection.Height());
   }
 
   // Reduce the rectangle side lengths in multiples of the dash period length
@@ -917,8 +917,8 @@ ShrinkClippedStrokedRect(const Rect &aStrokedRect, const IntRect &aDeviceClip,
   insetBy.left = RoundDownToMultiple(insetBy.left, dashPeriodLength);
 
   strokedRectDouble.Deflate(insetBy);
-  return Rect(strokedRectDouble.x,
-              strokedRectDouble.y,
+  return Rect(strokedRectDouble.X(),
+              strokedRectDouble.Y(),
               strokedRectDouble.Width(),
               strokedRectDouble.Height());
 }
@@ -1613,7 +1613,7 @@ DrawTarget::Draw3DTransformedSurface(SourceSurface* aSurface, const Matrix4x4& a
     return true;
   }
   // Offset the matrix by the transformed origin.
-  fullMat.PostTranslate(-xformBounds.x, -xformBounds.y, 0);
+  fullMat.PostTranslate(-xformBounds.X(), -xformBounds.Y(), 0);
 
   // Read in the source data.
   Maybe<MutexAutoLock> lock;
@@ -1926,7 +1926,7 @@ DrawTargetSkia::CopySurface(SourceSurface *aSurface,
   if (image->isAlphaOnly()) {
     mCanvas->clear(SK_ColorTRANSPARENT);
   }
-  mCanvas->drawImage(image, -SkIntToScalar(aSourceRect.x), -SkIntToScalar(aSourceRect.y), &paint);
+  mCanvas->drawImage(image, -SkIntToScalar(aSourceRect.X()), -SkIntToScalar(aSourceRect.Y()), &paint);
   mCanvas->restore();
 }
 
@@ -2131,6 +2131,7 @@ void
 DrawTargetSkia::PopClip()
 {
   mCanvas->restore();
+  SetTransform(GetTransform());
 }
 
 void
@@ -2138,7 +2139,15 @@ DrawTargetSkia::PushLayer(bool aOpaque, Float aOpacity, SourceSurface* aMask,
                           const Matrix& aMaskTransform, const IntRect& aBounds,
                           bool aCopyBackground)
 {
-  PushedLayer layer(GetPermitSubpixelAA(), aOpaque, aOpacity, aMask, aMaskTransform,
+  PushLayerWithBlend(aOpaque, aOpacity, aMask, aMaskTransform, aBounds, aCopyBackground, CompositionOp::OP_OVER);
+}
+
+void
+DrawTargetSkia::PushLayerWithBlend(bool aOpaque, Float aOpacity, SourceSurface* aMask,
+                                   const Matrix& aMaskTransform, const IntRect& aBounds,
+                                   bool aCopyBackground, CompositionOp aCompositionOp)
+{
+  PushedLayer layer(GetPermitSubpixelAA(), aOpaque, aOpacity, aCompositionOp, aMask, aMaskTransform,
                     mCanvas->getTopDevice());
   mPushedLayers.push_back(layer);
 
@@ -2147,6 +2156,9 @@ DrawTargetSkia::PushLayer(bool aOpaque, Float aOpacity, SourceSurface* aMask,
   // If we have a mask, set the opacity to 0 so that SkCanvas::restore skips
   // implicitly drawing the layer so that we can properly mask it in PopLayer.
   paint.setAlpha(aMask ? 0 : ColorFloatToByte(aOpacity));
+  if (!aMask) {
+    paint.setBlendMode(GfxOpToSkiaOp(layer.mCompositionOp));
+  }
 
   // aBounds is supplied in device space, but SaveLayerRec wants local space.
   SkRect bounds = IntRectToSkRect(aBounds);
@@ -2198,6 +2210,7 @@ DrawTargetSkia::PopLayer()
 
     SkPaint paint;
     paint.setAlpha(ColorFloatToByte(layer.mOpacity));
+    paint.setBlendMode(GfxOpToSkiaOp(layer.mCompositionOp));
 
     SkMatrix maskMat, layerMat;
     // Get the total transform affecting the mask, considering its pattern
@@ -2218,6 +2231,8 @@ DrawTargetSkia::PopLayer()
         paint.setColor(SK_ColorTRANSPARENT);
       }
 
+      maskMat.postTranslate(layer.mMask->GetRect().X(), layer.mMask->GetRect().Y());
+
       Maybe<MutexAutoLock> lock;
       sk_sp<SkImage> alphaMask = ExtractAlphaForSurface(layer.mMask, lock);
 
@@ -2241,6 +2256,7 @@ DrawTargetSkia::PopLayer()
     mCanvas->restore();
   }
 
+  SetTransform(GetTransform());
   SetPermitSubpixelAA(layer.mOldPermitSubpixelAA);
 
   mPushedLayers.pop_back();
