@@ -72,32 +72,61 @@ bool ScaledFontMac::sSymbolLookupDone = false;
 // Helper to create a CTFont from a CGFont, copying any variations that were
 // set on the original CGFont.
 static CTFontRef
-CreateCTFontFromCGFontWithVariations(CGFontRef aCGFont, CGFloat aSize)
+CreateCTFontFromCGFontWithVariations(CGFontRef aCGFont, CGFloat aSize,
+                                     bool aInstalledFont)
 {
     // Avoid calling potentially buggy variation APIs on pre-Sierra macOS
-    // versions (see bug 1331683)
-    if (!nsCocoaFeatures::OnSierraOrLater()) {
-        return CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr, nullptr);
-    }
+    // versions (see bug 1331683).
+    //
+    // And on HighSierra, CTFontCreateWithGraphicsFont properly carries over
+    // variation settings from the CGFont to CTFont, so we don't need to do
+    // the extra work here -- and this seems to avoid Core Text crashiness
+    // seen in bug 1454094.
+    //
+    // However, for installed fonts it seems we DO need to copy the variations
+    // explicitly even on 10.13, otherwise fonts fail to render (as in bug
+    // 1455494) when non-default values are used. Fortunately, the crash
+    // mentioned above occurs with data fonts, not (AFAICT) with system-
+    // installed fonts.
+    //
+    // So we only need to do this "the hard way" on Sierra, and for installed
+    // fonts on HighSierra+; otherwise, just let the standard CTFont function
+    // do its thing.
+    //
+    // NOTE in case this ever needs further adjustment: there is similar logic
+    // in four places in the tree (sadly):
+    //    CreateCTFontFromCGFontWithVariations in gfxMacFont.cpp
+    //    CreateCTFontFromCGFontWithVariations in ScaledFontMac.cpp
+    //    CreateCTFontFromCGFontWithVariations in cairo-quartz-font.c
+    //    ctfont_create_exact_copy in SkFontHost_mac.cpp
 
-    CFDictionaryRef vars = CGFontCopyVariations(aCGFont);
     CTFontRef ctFont;
-    if (vars) {
-        CFDictionaryRef varAttr =
-            CFDictionaryCreate(nullptr,
-                               (const void**)&kCTFontVariationAttribute,
-                               (const void**)&vars, 1,
-                               &kCFTypeDictionaryKeyCallBacks,
-                               &kCFTypeDictionaryValueCallBacks);
-        CFRelease(vars);
+    if (nsCocoaFeatures::OnSierraExactly() ||
+        (aInstalledFont && nsCocoaFeatures::OnHighSierraOrLater())) {
+        CFDictionaryRef vars = CGFontCopyVariations(aCGFont);
+        if (vars) {
+            CFDictionaryRef varAttr =
+                CFDictionaryCreate(nullptr,
+                                   (const void**)&kCTFontVariationAttribute,
+                                   (const void**)&vars, 1,
+                                   &kCFTypeDictionaryKeyCallBacks,
+                                   &kCFTypeDictionaryValueCallBacks);
+            CFRelease(vars);
 
-        CTFontDescriptorRef varDesc = CTFontDescriptorCreateWithAttributes(varAttr);
-        CFRelease(varAttr);
+            CTFontDescriptorRef varDesc =
+                CTFontDescriptorCreateWithAttributes(varAttr);
+            CFRelease(varAttr);
 
-        ctFont = CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr, varDesc);
-        CFRelease(varDesc);
+            ctFont = CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr,
+                                                  varDesc);
+            CFRelease(varDesc);
+        } else {
+            ctFont = CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr,
+                                                  nullptr);
+        }
     } else {
-        ctFont = CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr, nullptr);
+        ctFont = CTFontCreateWithGraphicsFont(aCGFont, aSize, nullptr,
+                                              nullptr);
     }
     return ctFont;
 }
@@ -105,13 +134,15 @@ CreateCTFontFromCGFontWithVariations(CGFontRef aCGFont, CGFloat aSize)
 ScaledFontMac::ScaledFontMac(CGFontRef aFont,
                              const RefPtr<UnscaledFont>& aUnscaledFont,
                              Float aSize,
+                             bool aOwnsFont,
                              const Color &aFontSmoothingBackgroundColor,
                              bool aUseFontSmoothing,
-                             bool aOwnsFont)
+                             bool aApplySyntheticBold)
   : ScaledFontBase(aUnscaledFont, aSize)
   , mFont(aFont)
   , mFontSmoothingBackgroundColor(aFontSmoothingBackgroundColor)
   , mUseFontSmoothing(aUseFontSmoothing)
+  , mApplySyntheticBold(aApplySyntheticBold)
 {
   if (!sSymbolLookupDone) {
     CTFontDrawGlyphsPtr =
@@ -126,7 +157,9 @@ ScaledFontMac::ScaledFontMac(CGFontRef aFont,
 
   if (CTFontDrawGlyphsPtr != nullptr) {
     // only create mCTFont if we're going to be using the CTFontDrawGlyphs API
-    mCTFont = CreateCTFontFromCGFontWithVariations(aFont, aSize);
+    auto unscaledMac = static_cast<UnscaledFontMac*>(aUnscaledFont.get());
+    bool dataFont = unscaledMac->IsDataFont();
+    mCTFont = CreateCTFontFromCGFontWithVariations(aFont, aSize, !dataFont);
   } else {
     mCTFont = nullptr;
   }
@@ -147,7 +180,10 @@ SkTypeface* ScaledFontMac::GetSkTypeface()
     if (mCTFont) {
       mTypeface = SkCreateTypefaceFromCTFont(mCTFont);
     } else {
-      CTFontRef fontFace = CreateCTFontFromCGFontWithVariations(mFont, mSize);
+      auto unscaledMac = static_cast<UnscaledFontMac*>(GetUnscaledFont().get());
+      bool dataFont = unscaledMac->IsDataFont();
+      CTFontRef fontFace =
+        CreateCTFontFromCGFontWithVariations(mFont, mSize, !dataFont);
       mTypeface = SkCreateTypefaceFromCTFont(fontFace);
       CFRelease(fontFace);
     }
@@ -322,6 +358,31 @@ UnscaledFontMac::GetFontFileData(FontFileDataOutput aDataCallback, void *aBaton)
     return true;
 }
 
+bool
+UnscaledFontMac::GetWRFontDescriptor(WRFontDescriptorOutput aCb, void* aBaton)
+{
+  if (mIsDataFont) {
+    return false;
+  }
+
+  CFStringRef psname = CGFontCopyPostScriptName(mFont);
+  if (!psname) {
+    return false;
+  }
+
+  char buf[256];
+  const char* cstr = CFStringGetCStringPtr(psname, kCFStringEncodingUTF8);
+  if (!cstr) {
+    if (!CFStringGetCString(psname, buf, sizeof(buf), kCFStringEncodingUTF8)) {
+      return false;
+    }
+    cstr = buf;
+  }
+
+  aCb(reinterpret_cast<const uint8_t*>(cstr), strlen(cstr), 0, aBaton);
+  return true;
+}
+
 static void
 CollectVariationsFromDictionary(const void* aKey, const void* aValue, void* aContext)
 {
@@ -362,27 +423,35 @@ GetVariationsForCTFont(CTFontRef aCTFont, std::vector<FontVariation>* aOutVariat
 bool
 ScaledFontMac::GetFontInstanceData(FontInstanceDataOutput aCb, void* aBaton)
 {
-    // Collect any variation settings that were incorporated into the CTFont.
-    std::vector<FontVariation> variations;
-    if (!GetVariationsForCTFont(mCTFont, &variations)) {
-      return false;
-    }
-    aCb(nullptr, 0, variations.data(), variations.size(), aBaton);
-    return true;
+  // Collect any variation settings that were incorporated into the CTFont.
+  std::vector<FontVariation> variations;
+  if (!GetVariationsForCTFont(mCTFont, &variations)) {
+    return false;
+  }
+  aCb(nullptr, 0, variations.data(), variations.size(), aBaton);
+  return true;
 }
 
 bool
-ScaledFontMac::GetWRFontInstanceOptions(Maybe<wr::WrFontInstanceOptions>* aOutOptions,
+ScaledFontMac::GetWRFontInstanceOptions(Maybe<wr::FontInstanceOptions>* aOutOptions,
                                         Maybe<wr::FontInstancePlatformOptions>* aOutPlatformOptions,
                                         std::vector<FontVariation>* aOutVariations)
 {
-    GetVariationsForCTFont(mCTFont, aOutVariations);
+  GetVariationsForCTFont(mCTFont, aOutVariations);
 
-    wr::FontInstancePlatformOptions platformOptions;
-    // FIXME with later webrender
-    // platformOptions.font_smoothing = mUseFontSmoothing;
-    *aOutPlatformOptions = Some(platformOptions);
-    return true;
+  wr::FontInstanceOptions options;
+  options.render_mode = wr::FontRenderMode::Subpixel;
+  options.subpx_dir = wr::SubpixelDirection::Horizontal;
+  options.flags = 0;
+  if (mUseFontSmoothing) {
+    options.flags |= wr::FontInstanceFlags::FONT_SMOOTHING;
+  }
+  if (mApplySyntheticBold) {
+    options.flags |= wr::FontInstanceFlags::SYNTHETIC_BOLD;
+  }
+  options.bg_color = wr::ToColorU(mFontSmoothingBackgroundColor);
+  *aOutOptions = Some(options);
+  return true;
 }
 
 static CFDictionaryRef
@@ -521,7 +590,7 @@ UnscaledFontMac::CreateScaledFont(Float aGlyphSize,
   }
 
   RefPtr<ScaledFontMac> scaledFont =
-    new ScaledFontMac(fontRef, this, aGlyphSize, Color(), true, fontRef != mFont);
+    new ScaledFontMac(fontRef, this, aGlyphSize, fontRef != mFont);
 
   if (!scaledFont->PopulateCairoScaledFont()) {
     gfxWarning() << "Unable to create cairo scaled Mac font.";
