@@ -40,7 +40,7 @@ class ClipExporter;
  * All drawing happens by creating a path and then stroking or filling it.
  * The functions like Rectangle and Arc do not do any drawing themselves.
  * When a path is drawn (stroked or filled), it is filled/stroked with a
- * pattern set by SetPattern, SetColor or SetSource.
+ * pattern set by SetPattern or SetColor.
  *
  * Note that the gfxContext takes coordinates in device pixels,
  * as opposed to app units.
@@ -50,6 +50,7 @@ class gfxContext final {
     typedef mozilla::gfx::CompositionOp CompositionOp;
     typedef mozilla::gfx::JoinStyle JoinStyle;
     typedef mozilla::gfx::FillRule FillRule;
+    typedef mozilla::gfx::Float Float;
     typedef mozilla::gfx::Path Path;
     typedef mozilla::gfx::Pattern Pattern;
     typedef mozilla::gfx::Rect Rect;
@@ -172,12 +173,14 @@ public:
     /**
      * Replaces the current transformation matrix with matrix.
      */
-    void SetMatrix(const gfxMatrix& matrix);
+    void SetMatrix(const mozilla::gfx::Matrix& matrix);
+    void SetMatrixDouble(const gfxMatrix& matrix);
 
     /**
      * Returns the current transformation matrix.
      */
-    gfxMatrix CurrentMatrix() const;
+    mozilla::gfx::Matrix CurrentMatrix() const;
+    gfxMatrix CurrentMatrixDouble() const;
 
     /**
      * Converts a point from device to user coordinates using the inverse
@@ -267,15 +270,6 @@ public:
      */
     void SetColor(const mozilla::gfx::Color& aColor);
 
-    /**
-     * Uses a surface for drawing. This is a shorthand for creating a
-     * pattern and setting it.
-     *
-     * @param offset from the source surface, to use only part of it.
-     *        May need to make it negative.
-     */
-    void SetSource(gfxASurface *surface, const gfxPoint& offset = gfxPoint(0.0, 0.0));
-
     /**
      * Uses a pattern for drawing.
      */
@@ -293,7 +287,7 @@ public:
      * Paints the current source surface/pattern everywhere in the current
      * clip region.
      */
-    void Paint(gfxFloat alpha = 1.0);
+    void Paint(Float alpha = 1.0);
 
     /**
      ** Painting with a Mask
@@ -310,25 +304,25 @@ public:
      ** Line Properties
      **/
 
-    void SetDash(gfxFloat *dashes, int ndash, gfxFloat offset);
+    void SetDash(const Float *dashes, int ndash, Float offset);
     // Return true if dashing is set, false if it's not enabled or the
     // context is in an error state.  |offset| can be nullptr to mean
     // "don't care".
-    bool CurrentDash(FallibleTArray<gfxFloat>& dashes, gfxFloat* offset) const;
+    bool CurrentDash(FallibleTArray<Float>& dashes, Float* offset) const;
     // Returns 0.0 if dashing isn't enabled.
-    gfxFloat CurrentDashOffset() const;
+    Float CurrentDashOffset() const;
 
     /**
      * Sets the line width that's used for line drawing.
      */
-    void SetLineWidth(gfxFloat width);
+    void SetLineWidth(Float width);
 
     /**
      * Returns the currently set line width.
      *
      * @see SetLineWidth
      */
-    gfxFloat CurrentLineWidth() const;
+    Float CurrentLineWidth() const;
 
     /**
      * Sets the line caps, i.e. how line endings are drawn.
@@ -343,8 +337,8 @@ public:
     void SetLineJoin(JoinStyle join);
     JoinStyle CurrentLineJoin() const;
 
-    void SetMiterLimit(gfxFloat limit);
-    gfxFloat CurrentMiterLimit() const;
+    void SetMiterLimit(Float limit);
+    Float CurrentMiterLimit() const;
 
     /**
      * Sets the operator used for all further drawing. The operator affects
@@ -468,7 +462,6 @@ private:
   typedef mozilla::gfx::DrawTarget DrawTarget;
   typedef mozilla::gfx::Color Color;
   typedef mozilla::gfx::StrokeOptions StrokeOptions;
-  typedef mozilla::gfx::Float Float;
   typedef mozilla::gfx::PathBuilder PathBuilder;
   typedef mozilla::gfx::SourceSurface SourceSurface;
 
@@ -487,10 +480,6 @@ private:
     mozilla::gfx::CompositionOp op;
     Color color;
     RefPtr<gfxPattern> pattern;
-    RefPtr<gfxASurface> sourceSurfCairo;
-    RefPtr<SourceSurface> sourceSurface;
-    mozilla::gfx::Point sourceSurfaceDeviceOffset;
-    Matrix surfTransform;
     Matrix transform;
     struct PushedClip {
       RefPtr<Path> path;
@@ -522,7 +511,6 @@ private:
   void EnsurePath();
   // This ensures mPathBuilder contains a valid PathBuilder (in user space!)
   void EnsurePathBuilder();
-  void FillAzure(const Pattern& aPattern, mozilla::gfx::Float aOpacity);
   CompositionOp GetOp();
   void ChangeTransform(const mozilla::gfx::Matrix &aNewMatrix, bool aUpdatePatternTransform = true);
   Rect GetAzureDeviceSpaceClipBounds() const;
@@ -628,7 +616,7 @@ public:
       }
     }
 
-    const gfxMatrix& Matrix()
+    const mozilla::gfx::Matrix& Matrix()
     {
       MOZ_ASSERT(mContext, "mMatrix doesn't contain a useful matrix");
       return mMatrix;
@@ -638,7 +626,7 @@ public:
 
 private:
     gfxContext *mContext;
-    gfxMatrix   mMatrix;
+    mozilla::gfx::Matrix mMatrix;
 };
 
 
