@@ -12,33 +12,29 @@ extern crate bitflags;
 extern crate byteorder;
 #[cfg(feature = "nightly")]
 extern crate core;
-extern crate euclid;
-extern crate fxhash;
-#[macro_use]
-extern crate heapsize;
-#[cfg(feature = "ipc")]
-extern crate ipc_channel;
-#[macro_use]
-extern crate serde;
-extern crate time;
-
 #[cfg(target_os = "macos")]
 extern crate core_foundation;
-
 #[cfg(target_os = "macos")]
 extern crate core_graphics;
-
 #[cfg(target_os = "windows")]
 extern crate dwrote;
+extern crate euclid;
+#[cfg(feature = "ipc")]
+extern crate ipc_channel;
+extern crate serde;
+#[macro_use]
+extern crate serde_derive;
+extern crate time;
+
 
-mod units;
 mod api;
-mod color;
 pub mod channel;
+mod color;
 mod display_item;
 mod display_list;
 mod font;
 mod image;
+mod units;
 
 pub use api::*;
 pub use color::*;
@@ -46,8 +42,4 @@ pub use display_item::*;
 pub use display_list::*;
 pub use font::*;
 pub use image::*;
-use std::collections::{HashMap, HashSet};
-use std::hash::BuildHasherDefault;
 pub use units::*;
-type FastHashMap<K, V> = HashMap<K, V, BuildHasherDefault<fxhash::FxHasher>>;
-type FastHashSet<T> = HashSet<T, BuildHasherDefault<fxhash::FxHasher>>;
